# Matlab default colors: https://nl.mathworks.com/help/matlab/creating_plots/specify-plot-colors.html
# Keep codes in hex format here, because FontFace do not accept rgb or rgba, (and we use it for text)
# for charts and plotly we do conversion to rgba in resolve_highlight_color

color_codes = {
    "red": "#FF0000",
    "r": "#FF0000",
    "green": "#00FF00",
    "g": "#00FF00",
    "blue": "#0000FF",
    "b": "#0000FF",
    "cyan": "#00FFFF",
    "c": "#00FFFF",
    "magenta": "#FF00FF",
    "m": "#FF00FF",
    "yellow": "#FFFF00",
    "y": "#FFFF00",
    "black": "#000000",
    "k": "#000000",
    "white": "#FFFFFF",
    "w": "#FFFFFF"
}
